#include <iostream>
#include <string>
#include <fstream>
#include <random>
#include <exception>
#include <eigen3/Eigen/Dense>

typedef Eigen::MatrixXd Matrix ;
typedef Eigen::VectorXd Vector ;


/*******************************************************************************
 * This function generate polyhedra for test
*******************************************************************************/
int main (int argc, const char* argv[]) {
  if(argc != 7) {
    std::cout << "usage: " << argv[0] << " <filename> <number_of_polyhedra> " 
      <<  "<total_number_of_constraints> <number_of_variables> <density (number of zeros) > " 
      << "<number_of_redundant_constraints>" << std::endl ; 
    return -1 ;
  }   
  int polyNum = std::stoi( argv[2] ) ;
  int totalConsNum = std::stoi( argv[3] ) ;
  int variNum = std::stoi( argv[4] ) ;
  int zeroNum = std::stoi( argv[5] ) ;
  int reConsNum = std::stoi( argv[6] ) ;   
  int consNum = totalConsNum - reConsNum ;
  if(zeroNum >= variNum) {
    std::cerr << "Error: the number of zeros is larger than the number of variables." << std::endl ;
    std::terminate() ;
  }
  //write the information to the file
  std::ofstream ofs(argv[1]) ;  
  if ( ! ofs.is_open() ) {
    std::cerr << "Cannot open file." << std::endl ;
    std::terminate() ; 
  }
  std::string line ;
  line = "polyhedra = " + std::to_string(polyNum) ;
  ofs << line << std::endl ;
  line = "constraints = " + std::to_string(totalConsNum) ;
  ofs << line << std::endl ;
  line = "redundancy = " + std::to_string(reConsNum) ;
  ofs << line << std::endl ;
  line = "variables = " + std::to_string(variNum) ;
  ofs << line << std::endl ; 
  line = "zeros = " + std::to_string(zeroNum) ;
  ofs << line << std::endl ;  
  ofs << std:: endl ;
  ofs << "Begin" << std::endl << std:: endl;
  // create random direction and distance
  // the constraints are Cx <= b
  Vector center(variNum) ;
  for(int i = 0; i < variNum; ++ i) {
    center(i) = 50.0 ;
  }
  std::random_device rd ;
  std::mt19937_64 gen( rd() ) ;
  for (int p = 0; p < polyNum; ++ p) {
    Matrix constraints(consNum, variNum) ;
    Vector constant(consNum) ;
    for(int i = 0; i < consNum; ++ i) {
      int j = 0 ;
      // generate C
      std::uniform_int_distribution<> disC(-50, 50) ;
      while(j < variNum) {
        int coef = disC(gen) ;
        if (coef == 0) continue ;
        else {
          constraints(i, j) = coef ;
          ++ j ;
        }
      }
      // generate b
      std::uniform_int_distribution<> disB(1, 50) ;
      constant(i) =  disB(gen) ;
      
    } 
    // set zeros according to the density
    if (zeroNum != 0) {
      for (int i = 0; i < consNum; ++ i) {
        int z = 0 ;
        std::vector<bool> isZero(variNum, false) ;
        std::uniform_int_distribution<> disIdx(0, variNum-1) ;
        while (z < zeroNum) { 
          int zeroIdx = disIdx(gen) ;  
          if (isZero[zeroIdx] == true) continue ;
          else {
            constraints(i, zeroIdx) = 0 ;
            isZero[zeroIdx] = true ;
            ++ z ; 
          }
        }  
      }
    }
    for (int i = 0; i < consNum; ++ i) {
      // test with the point
      double temp = center.dot( constraints.row(i) ) ;  
      if(temp - constant(i) > 0) {
        constraints.row(i) = -constraints.row(i) ;
        constant(i) = -constant(i) ;
      }
    }

    // create the redundant constraints
    std::uniform_int_distribution<> disLamda(1, 5) ;
    std::uniform_int_distribution<> disCons(0, consNum-1) ;
    Matrix constraints2(reConsNum, variNum) ;
    Vector constant2(reConsNum) ;
    for(int i = 0; i < reConsNum; ++ i) {
      int lamda1 = disLamda(gen) ;
      int lamda2 = disLamda(gen) ;
      int lamda3 = disLamda(gen) ;
      int idx1 = disCons(gen) ;
      int idx2 = disCons(gen) ;
      int idx3 = disCons(gen) ;
      constraints2.row(i) = lamda1 * constraints.row(idx1) + lamda2 * constraints.row(idx2) + lamda3 * constraints.row(idx3) ; 
      constant2(i) = lamda1 * constant(idx1) + lamda2 * constant(idx2) + lamda3 * constant(idx3) ;
    }
    std::string name = "P_" + std::to_string(polyNum) + "_" + std::to_string(consNum) + "_" 
      + std::to_string(variNum) + "_" + std::to_string(zeroNum) + "_" + std::to_string(p) ;
    ofs << name << std::endl ;
    for (int i = 0; i < consNum; ++ i) {
      for (int j = 0; j < variNum; ++ j) {
        ofs << constraints(i, j) << "\t" ;
      }
      ofs << constant(i) << std::endl ; 
    }
    for (int i = 0; i < reConsNum; ++ i) {
      for (int j = 0; j < variNum; ++ j) {
        ofs << constraints2(i, j) << "\t" ;
      }
      ofs << constant2(i) << std::endl ; 
    }
    ofs << std::endl ;
  }

  return 0 ;
}
