/*******************************************************************************
 * Copyright (c) 2016 Dec. Verimag. All rights reserved.
 * @author Hang YU
*******************************************************************************/

#include <iostream>
#include <fstream>
#include <regex>
#include <algorithm>
#include <string>
#include "ioInterface.h"

/*******************************************************************************
 * Read polyhedron from a file. The polyhedron is stored as matrix in that file.
*******************************************************************************/
std::vector<Polyhedron> IoInterface::LoadPolyhedra (const char* filepath) {
  std::vector<Polyhedron> polyVec ;
  std::ifstream ifs(filepath);
  if ( ! ifs.is_open() ) {
    std::cerr << "Cannot open file." << std::endl ;
    std::terminate() ;
  }
  std::string line ;
  std::string infoName ;
  int infoVal ;
  int pos ;
  int polyNum = 0 ;
  int consNum = 0 ;
  int variNum = 0 ;
  int zeroNum = 0 ; 
  std::regex pattern("[[:space:]]") ;
  while( std::getline(ifs, line) ) {
    if (line.length() == 0) continue ;
    line = std::regex_replace(line, pattern, "");
    if (line == "begin" || line == "Begin" || line == "BEGIN") break ;
    pos = line.find("=") ;
    infoName = line.substr(0, pos) ;
    infoVal = std::stoi(line.substr(pos+1));
    // tolower is a global function
    std::transform(infoName.begin(), infoName.end(), infoName.begin(), ::tolower);
    if (infoName == "polyhedra") {
      polyNum = infoVal ;
    }
    if (infoName == "constraints") {
      consNum = infoVal ;
    }
    if (infoName == "variables") {
      variNum = infoVal ;
    }
    if (infoName == "zeros") {
      zeroNum = infoVal ;
    }
  }

  if (polyNum == 0 || consNum == 0 || variNum == 0) {
    std::cerr << "Need information of the number of Polyhedra, constraints and variables" << std::endl ;
    std::terminate() ;
  }

  for (int i = 0; i < polyNum; ++ i) {
    // read the name of polyhedron
    while( std::getline(ifs, line) ) {
      line = std::regex_replace(line, pattern, "") ;
      if (line[0] >= 'a' && line[0] <= 'z') break ;
      else if (line[0] >= 'A' && line[0] <= 'Z') break ;
      else if (line[0] >= '0' && line[0] <= '9') {
        std::cerr << "The name of Polyhedron is missing or illegal." << std::endl ;
        std::terminate() ;
      }
      else if (line.length() != 0) {
        std::cerr << "The name of Polyhedron is illegal." << std::endl ;
        std::terminate() ;
      }
    }
    Polyhedron newPoly(consNum, variNum) ;
    double newVal ;
    for (int i = 0; i < consNum; i++) {
      for (int j = 0; j < variNum; j++) {
        ifs >> newVal ;
        newPoly.SetCoef(i, j, newVal) ;
      }
      ifs >> newVal ;
      newPoly.SetConstant(i, newVal) ;
    }
    polyVec.push_back(newPoly) ;
  }

  return polyVec ;
}
