/*******************************************************************************
 * Copyright (c) 2017 Jan. Verimag. All rights reserved.
 * @author Hang YU
*******************************************************************************/

#include <string>
#include <iostream>
#include "apronInterface.h"

/*******************************************************************************
 * See apronInterface.h
*******************************************************************************/
ApronInterface::ApronInterface (const Polyhedron& poly) {
  _man = pk_manager_alloc(true) ;
  const int variNum = poly.get_variable_num() ;
  const int consNum = poly.get_constraints_num() ;
  ap_var_t* name_of_vari = new ap_var_t[variNum] ; 
  std::vector<std::string> variName(variNum) ;
  for (int j = 0; j < variNum; ++ j) {
    variName[j] = "x" + std::to_string(j) ;
    name_of_vari[j] = (ap_var_t)( variName[j].c_str() ) ;
  }
  _env = ap_environment_alloc(NULL, 0, &name_of_vari[0], variNum) ;
  double val, constant ;
  ap_linexpr1_t expr ;
  ap_lincons1_t cons ;
  // set constraints
  _cons_array = ap_lincons1_array_make(_env, consNum) ; 
  for (int i = 0; i < consNum; ++ i) {
    expr = ap_linexpr1_make(_env, AP_LINEXPR_DENSE, 0) ;
    for (int j = 0; j < variNum; ++ j) {
      val = -poly.GetCoef(i, j) ;
      ap_linexpr1_set_coeff_scalar_double(&expr, name_of_vari[j], val) ;
    }
    constant = poly.GetConstant(i) ;
    ap_linexpr1_set_cst_scalar_double(&expr, constant) ;
    cons = ap_lincons1_make(AP_CONS_SUPEQ, &expr, NULL) ;
    ap_lincons1_array_set(&_cons_array, i, &cons) ;
  }
  _abs = ap_abstract1_of_lincons_array(_man, _env, &_cons_array) ;
  _generator = ap_abstract1_to_generator_array(_man, &_abs) ;
  delete[] name_of_vari ;
}

ApronInterface::~ApronInterface () {
  ap_lincons1_array_clear(&_cons_array) ;
  ap_generator1_array_clear(&_generator) ;
  ap_abstract1_clear(_man, &_abs) ;
  ap_environment_free(_env) ;
  ap_manager_free(_man) ;
}

/*******************************************************************************
 * @return true if the two polyhedra contain the same constraints
*******************************************************************************/
bool ApronInterface::CompareAbstract (const ApronInterface& apron) {
  ap_abstract1_t apronAbs = apron.get_abs() ;
  return ap_abstract1_is_eq( _man, &_abs, &apronAbs) ;  
}

/*******************************************************************************
 * @return the number of generators
*******************************************************************************/
int ApronInterface::GetGeneratorNum () {
  int size = (int)ap_generator1_array_size(&_generator) ;
  return size ;
}

ap_abstract1_t ApronInterface::get_abs () const {
  return _abs ;
}


/*******************************************************************************
 * Print the irredundant constraints
*******************************************************************************/
void ApronInterface::Print () {
  ap_abstract1_fprint(stdout, _man, &_abs) ;
}
