#include <iostream>
#include <ctime>
#include "raytracing.h"
#include "glpkInterface.h"

int main(int argc, char* argv[]) {
  if(argc != 2) {
    std::cerr << "usage: " << argv[0] << " filename" << std::endl ;
    return -1 ; 
  }
  Polyhedron poly = GlpkInterface::LoadLP(argv[1]) ;  
  poly.set_central_point( GlpkInterface::GetCentralPoint(poly) ) ;
  if(poly.get_central_point().IsEmpty()) {
    std::cerr << "Cannot find a point inside the polyhedra" << std::endl ;
    return -1 ;
  }
  // std::cout << "Found point: " << std::endl << poly.get_central_point() << std::endl ;
  time_t beginP = clock() ;
  poly.Minimize() ; 
  time_t endP = clock() ;
  double runtimeP = (endP - beginP) * 1000 / CLOCKS_PER_SEC ;
  std::cout << "Running time of parallel version: " << runtimeP << " ms" << std::endl ;
  poly.PrintActiveIdx() ;
  std::cout << std::endl ; 
  
  Polyhedron polyT = GlpkInterface::LoadLP(argv[1]) ;  
  polyT.set_central_point( GlpkInterface::GetCentralPoint(polyT) ) ;
  if(polyT.get_central_point().IsEmpty()) {
    std::cerr << "Cannot find a point inside the polyhedra" << std::endl ;
    return -1 ;
  }
  time_t beginT = clock() ;
  polyT.Minimize(2) ; 
  time_t endT = clock() ;
  double runtimeT = (endT - beginT) * 1000 / CLOCKS_PER_SEC ;
  std::cout << "Running time of two direction version: " << runtimeT << " ms" << std::endl ;
  polyT.PrintActiveIdx() ;
  std::cout << std::endl ; 

  Polyhedron polyM = GlpkInterface::LoadLP(argv[1]) ;  
  polyM.set_central_point( GlpkInterface::GetCentralPoint(polyM) ) ;
  if(polyM.get_central_point().IsEmpty()) {
    std::cerr << "Cannot find a point inside the polyhedra" << std::endl ;
    return -1 ;
  }
  time_t beginM = clock() ;
  polyM.Minimize(3) ; 
  time_t endM = clock() ;
  double runtimeM = (endM - beginM) * 1000 / CLOCKS_PER_SEC ;
  std::cout << "Running time of matrix version: " << runtimeM << " ms" << std::endl ;
  polyM.PrintActiveIdx() ;
  std::cout << std::endl ; 

  Polyhedron polyMT = GlpkInterface::LoadLP(argv[1]) ;  
  polyMT.set_central_point( GlpkInterface::GetCentralPoint(polyMT) ) ;
  if(polyMT.get_central_point().IsEmpty()) {
    std::cerr << "Cannot find a point inside the polyhedra" << std::endl ;
    return -1 ;
  }
  time_t beginMT = clock() ;
  polyMT.Minimize(4) ; 
  time_t endMT = clock() ;
  double runtimeMT = (endMT - beginMT) * 1000 / CLOCKS_PER_SEC ;
  std::cout << "Running time of matrix version in two direction: " << runtimeMT << " ms" << std::endl ;
  polyMT.PrintActiveIdx() ;
  
  return 0 ;
}
