#include <iostream>
#include <vector>
#include <ctime>
#include <fstream>
#include <chrono>
#include "ioInterface.h"
#include "glpkInterface.h"

int main(int argc, char* argv[]) {
  if(argc != 3 && argc != 4) {
    std::cerr << "usage: " << argv[0] << " <filename> <output path> [suffix]" << std::endl ;
    return -1 ; 
  }
  std::vector<Polyhedron> polyVec = IoInterface::LoadPolyhedra(argv[1]) ; 
  std::string filepath = std::string(argv[1]) ;
  std::size_t pos = filepath.find_last_of("/") ;
  std::string filename = filepath.substr(pos+1) ; 
  pos = filename.find_first_of(".") ;
  filename = filename.substr(0, pos) ;
  std::string outpath ;
  if (argc == 4) { 
    std::string suffix = argv[3] ;
    outpath = std::string(argv[2]) + "/" + filename + suffix + ".csv" ;
  }
  else {
    outpath = std::string(argv[2]) + "/" + filename + ".csv" ;
  }
  std::cout<<outpath << std::endl ;
  std::ofstream ofs(outpath) ; 
  if ( ! ofs.is_open() ) {
    std::cerr << "Cannot open file." << std::endl ;
    return -1 ;
  }
  ofs << "Polyhedron index, running time" << std::endl ;
  double totalTime = 0 ;
  int idx = 0 ;
  for (auto poly : polyVec) {
    auto start = std::chrono::steady_clock::now() ;
    poly.MinimizeSimple() ;
    auto end = std::chrono::steady_clock::now() ;
    std::chrono::duration<double> diff = (end - start) * 1000 ;
    std::cout << "Total running time: " << diff.count() << " ms" << std::endl ;
    ofs <<  idx << ", " << diff.count() << std::endl ;
    ++ idx ;
    totalTime += diff.count() ;
    poly.PrintActiveIdx() ;
  }
  double aveTime = totalTime / polyVec.size() ;
  std::cout << "Average running time: " << aveTime << " ms" << std::endl ;
  ofs << aveTime << std::endl ; 

  return 0 ;
} 
