/*
gcc -std=c99 -pedantic -Wall -L/usr/lib/openblas-base blas3.c -o blas3 -lblas
*/

#include <stdio.h>

extern void dgemm_(const char *transA,
		   const char *transB,
		   const int *m,
		   const int *n,
		   const int *k,
		   const double *alpha,
		   const double *A,
		   const int *ldA,
		   const double *B,
		   const int *ldB,
		   const double *beta,
		   double *restrict C,
		   const int *ldC);

extern void sgemm_(const char *transA,
		   const char *transB,
		   const int *m,
		   const int *n,
		   const int *k,
		   const float *alpha,
		   const float *A,
		   const int *ldA,
		   const float *B,
		   const int *ldB,
		   const float *beta,
		   float *restrict C,
		   const int *ldC);

int main() {
  const int m = 4, n = 5, k = 3;
  {
    const double a[3][4] = {{10,0,2,1},{2,1,2,1},{4,2,1,3}};
    const double b[5][3] = {{1,2,3},{6,5,4},{7,9,7},{1,1,1},{3,4,3}};
    const double alpha = 1.0, beta = 0.0;
    double c[5][4];
    dgemm_("N", "N", &m, &n, &k, &alpha, &a[0][0], &m, &b[0][0], &k, &beta, &c[0][0], &m);
    printf("Double precision\n");
    for(int j=0; j<n; j++) {
      for(int i=0; i<m; i++) {
	printf("%7.f ", c[j][i]);
      }
      printf("\n");
    }
  }
  {
    const float a[3][4] = {{10,0,2,1},{2,1,2,1},{4,2,1,3}};
    const float b[5][3] = {{1,2,3},{6,5,4},{7,9,7},{1,1,1},{3,4,3}};
    const float alpha = 1.0, beta = 0.0;
    float c[5][4];
    sgemm_("N", "N", &m, &n, &k, &alpha, &a[0][0], &m, &b[0][0], &k, &beta, &c[0][0], &m);
    printf("Single precision\n");
    for(int j=0; j<n; j++) {
      for(int i=0; i<m; i++) {
	printf("%7.f ", c[j][i]);
      }
      printf("\n");
    }
  }
  return 0;
}
