/*
gcc -std=c99 -pedantic -Wall -L/usr/lib/openblas-base blas3_complex.c -o blas3_complex -lblas
*/

#include <stdio.h>
#include <complex.h>

extern void zgemm_(const char *transA,
		   const char *transB,
		   const int *m,
		   const int *n,
		   const int *k,
		   const double complex *alpha,
		   const double complex *A,
		   const int *ldA,
		   const double complex *B,
		   const int *ldB,
		   const double complex *beta,
		   double complex *restrict C,
		   const int *ldC);

extern void cgemm_(const char *transA,
		   const char *transB,
		   const int *m,
		   const int *n,
		   const int *k,
		   const float complex *alpha,
		   const float complex *A,
		   const int *ldA,
		   const float complex *B,
		   const int *ldB,
		   const float complex *beta,
		   float complex *restrict C,
		   const int *ldC);

int main() {
  const int m = 4, n = 5, k = 3;
  {
    const double complex a[3][4] = {{10,0,2,1+6*I},{2,1,2+I,1},{4,2+33*I,1,3}};
    const double complex b[5][3] = {{1,2+9*I,3},{6+7*I,5,4},{7,9+3*I,7},{1,1,1},{3,4+I,3}};
    const double complex alpha = 1.0, beta = 0.0;
    double complex c[5][4];
    zgemm_("N", "N", &m, &n, &k, &alpha, &a[0][0], &m, &b[0][0], &k, &beta, &c[0][0], &m);
    printf("Double precision\n");
    for(int j=0; j<n; j++) {
      for(int i=0; i<m; i++) {
	printf("%7.f%+7.fi", creal(c[j][i]), cimag(c[j][i]));
      }
      printf("\n");
    }
  }
  {
    const float complex a[3][4] = {{10,0,2,1+6*I},{2,1,2+I,1},{4,2+33*I,1,3}};
    const float complex b[5][3] = {{1,2+9*I,3},{6+7*I,5,4},{7,9+3*I,7},{1,1,1},{3,4+I,3}};
    const float complex alpha = 1.0, beta = 0.0;
    float complex c[5][4];
    cgemm_("N", "N", &m, &n, &k, &alpha, &a[0][0], &m, &b[0][0], &k, &beta, &c[0][0], &m);
    printf("Single precision\n");
    for(int j=0; j<n; j++) {
      for(int i=0; i<m; i++) {
	printf("%7.f%+7.fi", crealf(c[j][i]), cimagf(c[j][i]));
      }
      printf("\n");
    }
  }
  return 0;
}
