#include <iostream>
#include <coin/ClpSimplex.hpp>
#include <coin/ClpLinearObjective.hpp>

static inline std::string toString(ClpSimplex::Status x) {
  switch (x) {
  case ClpSimplex::isFree : return "isFree";
  case ClpSimplex::basic : return "basic";
  case ClpSimplex::atUpperBound : return "atUpperBound";
  case ClpSimplex::atLowerBound : return "atLowerBound";
  case ClpSimplex::superBasic : return "superBasic";
  case ClpSimplex::isFixed : return "isFixed";
  default: std::terminate();
  }
}

int main() {
  #pragma omp parallel for
  for(int i=0; i<10000; i++) {
    ClpSimplex lp;
    lp.setLogLevel(0);
    std::vector<std::string> varNames;
    varNames.push_back("x");
    varNames.push_back("y");
    lp.resize(0, varNames.size());
    lp.copyColumnNames(varNames, 0, varNames.size());
    {
      const int nVars = 2;
      int vars[nVars] = { 0, 1 };
      double coeffs[nVars] = { 2.0, 1.0 };
      lp.addRow(nVars, vars, coeffs, -COIN_DBL_MAX, 10.0);
    }
    {
      const int nVars = 2;
      int vars[nVars] = { 0, 1 };
      double coeffs[nVars] = { 1.0, 0.5 };
      lp.addRow(nVars, vars, coeffs, -COIN_DBL_MAX, 3.0);
    }
    {
      const int nVars = 2;
      int vars[nVars] = { 0, 1 };
      double coeffs[nVars] = { 0.5, 1.0 };
      lp.addRow(nVars, vars, coeffs, -COIN_DBL_MAX, 2.0);
    }
    {
      const int nVars = 2;
      int vars[nVars] = { 0, 1 };
      double coeffs[nVars] = { 1.0, 5.0 };
      lp.addRow(nVars, vars, coeffs, -COIN_DBL_MAX, 6.000001);
      /* 6.0000010 basic
	 6.0000001 atUpperBound */
    }
    {
      lp.setObjectiveCoefficient(0, -1.0);
      lp.setObjectiveCoefficient(1, -1.0);
    }
    lp.primal();

#if 0
#pragma omp critical(printout)
    {
      std::cout
	<< "rows=" << lp.getNumRows()
	<< " columns=" << lp.getNumCols() << std::endl;
    
      double *solution = lp.primalColumnSolution();
      std::cout << "x=" << solution[0] << " y=" << solution[1] << std::endl;
      
      for(int i=0; i<lp.getNumRows(); i++) {
	std::cout << "constraint " << i << ": " << toString(lp.getRowStatus(i)) << std::endl;
      }
      std::cout << "largest primal error: " << lp.largestPrimalError() << std::endl;
      std::cout << "primal tolerance: " << lp.currentPrimalTolerance() << std::endl;
    }
#endif
  }
}
