#include <stdlib.h>
#include <stdio.h>
#include <cudd.h>
#include <assert.h>

static int variable(int nPigeons,
		    int nHoles,
		    int pigeon,
		    int hole) {
  assert(pigeon >= 0);
  assert(pigeon < nPigeons);
  assert(hole >= 0);
  assert(hole < nHoles);
  return pigeon + nPigeons*hole;
}

DdNode *gen_pigeons(DdManager *manager,
		    int nPigeons,
		    int nHoles) {
  
  DdNode *overall = Cudd_ReadOne(manager);
  Cudd_Ref(overall);
  
  /* each pigeon must be in a hole */
  for(int pigeon=0; pigeon<nPigeons; pigeon++) {
    DdNode *cond = Cudd_ReadOne(manager);
    Cudd_Ref(cond);
    for(int hole=0; hole<nHoles; hole++) {
      DdNode *var = Cudd_bddIthVar(manager,
				   variable(nPigeons, nHoles, pigeon, hole)),
	*tmp;
      tmp = Cudd_bddAnd(manager, Cudd_Not(var), cond);
      Cudd_Ref(tmp);
      Cudd_RecursiveDeref(manager,cond);
      cond = tmp;
    }
    {
      DdNode *tmp = Cudd_bddAnd(manager, overall, Cudd_Not(cond));
      Cudd_Ref(tmp);
      Cudd_RecursiveDeref(manager, overall);
      Cudd_RecursiveDeref(manager, cond);
      overall = tmp;
    }
  }

  /* each hole cannot contain two pigeons */
  for(int hole=0; hole<nHoles; hole++) {
    for(int pigeon1=0; pigeon1<nPigeons; pigeon1++) {
      for(int pigeon2=pigeon1+1; pigeon2<nPigeons; pigeon2++) {
	DdNode *var1, *var2, *cond, *tmp;
	var1 = Cudd_bddIthVar(manager,
			      variable(nPigeons, nHoles, pigeon1, hole));
	Cudd_Ref(var1);
	var2 = Cudd_bddIthVar(manager,
			      variable(nPigeons, nHoles, pigeon2, hole));
	Cudd_Ref(var2);
	cond = Cudd_bddAnd(manager, var1, var2);
	Cudd_Ref(cond);
	Cudd_RecursiveDeref(manager, var1);
	Cudd_RecursiveDeref(manager, var2);
	tmp = Cudd_bddAnd(manager, overall, Cudd_Not(cond));
	Cudd_Ref(tmp);
	Cudd_RecursiveDeref(manager, cond);
	Cudd_RecursiveDeref(manager, overall);
	overall = tmp;
      }
    }
  }
  
  return overall;
}

static int test_zero(DdManager *manager, DdNode *bdd) {
  return Cudd_Not(Cudd_ReadOne(manager)) == bdd;
}

int main() {
#pragma omp parallel
  {
    DdManager *manager = Cudd_Init(0,0,CUDD_UNIQUE_SLOTS,CUDD_CACHE_SLOTS,0);
#pragma omp for schedule (dynamic)
    for(int nHoles=0; nHoles <= 13; nHoles++) {
      int nPigeons = nHoles+1;
      Cudd_ResetStartTime(manager);
#if 0
      printf("Cudd_ReadSize: %d\n", Cudd_ReadSize(manager));
      printf("Cudd_ReadKeys: %u\n", Cudd_ReadKeys(manager));
      printf("Cudd_ReadDead: %u\n", Cudd_ReadDead(manager));
#endif
#pragma omp critical (printf)
      {
	
	fflush(stdout);
      }
      DdNode *pigeon_bdd = gen_pigeons(manager, nPigeons, nHoles);
      assert(test_zero(manager, pigeon_bdd));
#if 0
      printf("is zero: %d\n", test_zero(manager, pigeon_bdd));
      printf("Cudd_ReadSize: %d\n", Cudd_ReadSize(manager));
      printf("Cudd_ReadKeys: %u\n", Cudd_ReadKeys(manager));
      printf("Cudd_ReadDead: %u\n", Cudd_ReadDead(manager));
#endif
      Cudd_RecursiveDeref(manager, pigeon_bdd);
#if 0
      printf("deallocated\n");
      printf("Cudd_ReadSize: %d\n", Cudd_ReadSize(manager));
      printf("Cudd_ReadKeys: %u\n", Cudd_ReadKeys(manager));
      printf("Cudd_ReadDead: %u\n", Cudd_ReadDead(manager));
#endif
#pragma omp critical (printf)
      {
        printf("%d %d %lu\n", nPigeons, nHoles, Cudd_ReadElapsedTime(manager));
	fflush(stdout);
      }
    }
    Cudd_Quit(manager);
  }
  return 0;
}
