// g++ -I/usr/include/lpsolve/ -Wall -std=c++11 call_lp_solve.cpp -o call_lp_solver -l lp_solve

#include <cstdio>
#include "lp_solve.hpp"


int main() {
  const int VARIABLES=2;
  lp_solve::LP prog(0, VARIABLES);
  
  prog.set_verbose(lp_solve::IMPORTANT);

  prog.set_col_name(1, "x");
  prog.set_col_name(2, "y");
  prog.set_add_rowmode(lp_solve::TRUE);

  const lp_solve::REAL obj[VARIABLES+1] = {0, -1.0, -1.0};
  const lp_solve::REAL row1[VARIABLES+1] = {0, 2.0, 1.0};
  const lp_solve::REAL row2[VARIABLES+1] = {0, 1.0, 0.5};
  const lp_solve::REAL row3[VARIABLES+1] = {0, 0.5, 1.0};

  prog.set_obj_fn(obj);
  prog.add_constraint(row1, lp_solve::LE, 10.0);
  prog.add_constraint(row2, lp_solve::LE, 3.0);
  prog.add_constraint(row3, lp_solve::LE, 2.0);
  prog.set_add_rowmode(lp_solve::FALSE);

  if (! prog.write_lp("example.lp")) {
    fprintf(stderr, "cannot write example.lp\n");
    return 1;
  }
  
  int answer = prog.solve();
  printf("answer = %d\n", answer);

  lp_solve::REAL sol[VARIABLES];
  prog.get_variables(sol);
  printf("x=%f y=%f\n", sol[0], sol[1]);

  {
    int nbasis = prog.get_Nrows() + prog.get_Ncolumns();
    int basis[nbasis+1];
    printf("rows=%d columns=%d\n",
	   prog.get_Nrows(),
	   prog.get_Ncolumns());
    if (prog.get_basis(basis, lp_solve::TRUE)) {
      printf("basis:");
      for(int i=0; i<=nbasis; i++) {
	printf(" %d", basis[i]);
      }
      printf("\n");
    } else {
      printf("no basis\n");
    }
  }
  return 0;
}
