#ifndef _LP_SOLVE_HPP
#define _LP_SOLVE_HPP

namespace lp_solve {
#include "lp_solve_nicer.h"

enum error_code {
  SUCCESS = 1,
  ERROR = 0
};
  
class LP {
  lprec * const rec;

public:
  LP(int rows, int columns) :
    rec(make_lp(rows, columns)) {
  }

  ~LP() {
    delete_lp(rec);
  }

  lprec* get() const {
    return rec;
  }

  void set_verbose(verbosity level) {
    lp_solve::set_verbose(get(), level);
  }

  error_code set_col_name(int column, const char *new_name) {
    return (error_code) lp_solve::set_col_name(get(), column, new_name);
  }

  error_code set_add_rowmode(bool turnon) {
    return (error_code) lp_solve::set_add_rowmode(get(), turnon);
  }
  
  error_code set_obj_fn(const REAL* row) {
    return (error_code) lp_solve::set_obj_fn(get(), row);
  }
  
  error_code add_constraint(const REAL* row, constraint_type op, REAL rh) {
    return (error_code) lp_solve::add_constraint(get(), row, op, rh);
  }

  solve_status solve() {
    return (solve_status) lp_solve::solve(get());
  }
  
  error_code get_variables(REAL *sol) const {
    return (error_code) lp_solve::get_variables(get(), sol);
  }
  
  error_code get_basis(int *bascolumn, bool nonbasic) const {
    return (error_code) lp_solve::get_basis(get(), bascolumn, nonbasic);
  }
  
  error_code write_lp(const char* filename) const {
    return (error_code) lp_solve::write_lp(get(), filename);
  }

  int get_Nrows() const {
    return lp_solve::get_Nrows(get());
  }
  
  int get_Ncolumns() const {
    return lp_solve::get_Ncolumns(get());
  }
};  
}
#endif
