#ifndef __LP_SOLVE_NICER_H
#define __LP_SOLVE_NICER_H

struct lprec;

enum verbosity {
  NEUTRAL= 0, /* Only some specific debug messages in de debug print routines are reported */
  CRITICAL= 1, /* Only critical messages are reported. Hard errors like instability, out of memory, ... */
  SEVERE= 2,	/* Only severe messages are reported. Errors. */
  IMPORTANT= 3,	/* Only important messages are reported. Warnings and Errors. */
  NORMAL= 4,	/* Normal messages are reported. This is the default. */
  DETAILED= 5,  /* Detailed messages are reported. Like model size, continuing B&B improvements, ... */
  FULL= 6  
};

enum constraint_type {
  LE = 1,
  EQ = 3,
  GE = 2
};

enum solve_status {
  NOMEMORY = -2,
  OPTIMAL = 0,
  SUBOPTIMAL = 1,
  INFEASIBLE = 2,
  UNBOUNDED = 3,
  DEGENERATE = 4,
  NUMFAILURE = 5,
  USERABORT = 6,
  TIMEOUT = 7,
  RUNNING = 8,
  PRESOLVED = 9
};

enum flag {
  FALSE = 0,
  TRUE = 1
};
  
typedef double REAL;

#ifdef __cplusplus
extern "C" {
#endif

struct lprec *make_lp(int rows, int columns);
void delete_lp(lprec *lp);
  
void set_verbose(struct lprec*, enum verbosity level);
unsigned char set_col_name(struct lprec*, int column, const char *new_name);
unsigned char set_add_rowmode(struct lprec*, unsigned char turnon);
unsigned char set_obj_fn(struct lprec*, const REAL* row);
unsigned char add_constraint(struct lprec*, const REAL* row, enum constraint_type op, REAL rh);

unsigned char solve(struct lprec*);
unsigned char get_variables(struct lprec*, REAL *sol);
unsigned char get_basis(lprec *lp, int *bascolumn, unsigned char nonbasic);
unsigned char write_lp(lprec *lp, const char* filename);

int get_Nrows(const lprec *lp);
int get_Ncolumns(const lprec *lp);

#ifdef __cplusplus
}
#endif

#endif
